@@ -8,41 +8,25 @@ import select
 import threading
 import weakref
 import warnings
+from inspect import iscoroutinefunction
 from functools import wraps
 
-import imp
-
 from twisted.python import threadable
 from twisted.python.runtime import platform
 from twisted.python.failure import Failure
 from twisted.python.log import PythonLoggingObserver, err
-from twisted.internet.defer import maybeDeferred
+from twisted.internet.defer import maybeDeferred, ensureDeferred
 from twisted.internet.task import LoopingCall
 
+import wrapt
+
 from ._util import synchronized
 from ._resultstore import ResultStore
 
 _store = ResultStore()
 
 
-if hasattr(weakref, "WeakSet"):
-    WeakSet = weakref.WeakSet
-else:
-    class WeakSet(object):
-        """
-        Minimal WeakSet emulation.
-        """
-        def __init__(self):
-            self._items = weakref.WeakKeyDictionary()
-
-        def add(self, value):
-            self._items[value] = True
-
-        def __iter__(self):
-            return iter(self._items)
-
-
-class TimeoutError(Exception):
+class TimeoutError(Exception):  # pylint: disable=redefined-builtin
     """
     A timeout has been hit.
     """
@@ -67,8 +51,9 @@ class ResultRegistry(object):
        ReactorStopped exception to unblock any remaining EventualResult.wait()
        calls.
     """
-    def __init__(self, reactor):
-        self._results = WeakSet()
+
+    def __init__(self):
+        self._results = weakref.WeakSet()
         self._stopped = False
         self._lock = threading.Lock()
 
@@ -127,6 +112,7 @@ class EventualResult(object):
         Should only be run in Twisted thread, and only called once.
         """
         self._deferred = deferred
+
         # Because we use __del__, we need to make sure there are no cycles
         # involving this object, which is why we use a weakref:
         def put(result, eventual=weakref.ref(self)):
@@ -135,6 +121,7 @@ class EventualResult(object):
                 eventual._set_result(result)
             else:
                 err(result, "Unhandled error in EventualResult")
+
         deferred.addBoth(put)
 
     def _set_result(self, result):
@@ -168,7 +155,7 @@ class EventualResult(object):
         """
         self._reactor.callFromThread(lambda: self._deferred.cancel())
 
-    def _result(self, timeout=None):
+    def _result(self, timeout):
         """
         Return the result, if available.
 
@@ -181,11 +168,6 @@ class EventualResult(object):
         returned on one call, additional calls will return/raise the same
         result.
         """
-        if timeout is None:
-            warnings.warn("Unlimited timeouts are deprecated.",
-                          DeprecationWarning, stacklevel=3)
-            # Queue.get(None) won't get interrupted by Ctrl-C...
-            timeout = 2 ** 31
         self._result_set.wait(timeout)
         # In Python 2.6 we can't rely on the return result of wait(), so we
         # have to check manually:
@@ -194,7 +176,7 @@ class EventualResult(object):
         self._result_retrieved = True
         return self._value
 
-    def wait(self, timeout=None):
+    def wait(self, timeout):
         """
         Return the result, or throw the exception if result is a failure.
 
@@ -211,21 +193,6 @@ class EventualResult(object):
             raise RuntimeError(
                 "EventualResult.wait() must not be run in the reactor thread.")
 
-        if imp.lock_held():
-            try:
-                imp.release_lock()
-            except RuntimeError:
-                # The lock is held by some other thread. We should be safe
-                # to continue.
-                pass
-            else:
-                # If EventualResult.wait() is run during module import, if the
-                # Twisted code that is being run also imports something the result
-                # will be a deadlock. Even if that is not an issue it would
-                # prevent importing in other threads until the call returns.
-                raise RuntimeError(
-                    "EventualResult.wait() must not be run at module import time.")
-
         result = self._result(timeout)
         if isinstance(result, Failure):
             result.raiseException()
@@ -266,10 +233,17 @@ class ThreadLogObserver(object):
 
     In particular, used to wrap PythonLoggingObserver, so that blocking
     logging.py Handlers don't block the event loop.
+
+    Once Python 3.6 support is dropped, this can use a queue.SimpleQueue object
+    instead of a whole 'nother event loop.
     """
+
     def __init__(self, observer):
         self._observer = observer
-        if getattr(select, "poll", None):
+        if getattr(select, "epoll", None):
+            from twisted.internet.epollreactor import EPollReactor
+            reactorFactory = EPollReactor
+        elif getattr(select, "poll", None):
             from twisted.internet.pollreactor import PollReactor
             reactorFactory = PollReactor
         else:
@@ -277,8 +251,8 @@ class ThreadLogObserver(object):
             reactorFactory = SelectReactor
         self._logWritingReactor = reactorFactory()
         self._logWritingReactor._registerAsIOThread = False
-        self._thread = threading.Thread(target=self._reader,
-                                        name="CrochetLogWriter")
+        self._thread = threading.Thread(
+            target=self._reader, name="CrochetLogWriter")
         self._thread.start()
 
     def _reader(self):
@@ -298,17 +272,31 @@ class ThreadLogObserver(object):
         """
         A log observer that writes to a queue.
         """
-        self._logWritingReactor.callFromThread(self._observer, msg)
+
+        def log():
+            try:
+                self._observer(msg)
+            except Exception:
+                # Lower-level logging system blew up, nothing we can do, so
+                # just drop on the floor.
+                pass
+
+        self._logWritingReactor.callFromThread(log)
 
 
 class EventLoop(object):
     """
     Initialization infrastructure for running a reactor in a thread.
     """
-    def __init__(self, reactorFactory, atexit_register,
-                 startLoggingWithObserver=None,
-                 watchdog_thread=None,
-                 reapAllProcesses=None):
+
+    def __init__(
+        self,
+        reactorFactory,
+        atexit_register,
+        startLoggingWithObserver=None,
+        watchdog_thread=None,
+        reapAllProcesses=None
+    ):
         """
         reactorFactory: Zero-argument callable that returns a reactor.
         atexit_register: atexit.register, or look-alike.
@@ -340,7 +328,7 @@ class EventLoop(object):
         """
         self._started = True
         self._reactor = self._reactorFactory()
-        self._registry = ResultRegistry(self._reactor)
+        self._registry = ResultRegistry()
         # We want to unblock EventualResult regardless of how the reactor is
         # run, so we always register this:
         self._reactor.addSystemEventTrigger(
@@ -364,6 +352,7 @@ class EventLoop(object):
             self._reactor.callFromThread(self._startReapingProcesses)
         if self._startLoggingWithObserver:
             observer = ThreadLogObserver(PythonLoggingObserver().emit)
+
             def start():
                 # Twisted is going to override warnings.showwarning; let's
                 # make sure that has no effect:
@@ -372,18 +361,18 @@ class EventLoop(object):
                 log.showwarning = warnings.showwarning
                 self._startLoggingWithObserver(observer, False)
                 log.showwarning = original
+
             self._reactor.callFromThread(start)
 
             # We only want to stop the logging thread once the reactor has
             # shut down:
-            self._reactor.addSystemEventTrigger("after", "shutdown",
-                                                observer.stop)
+            self._reactor.addSystemEventTrigger(
+                "after", "shutdown", observer.stop)
         t = threading.Thread(
             target=lambda: self._reactor.run(installSignalHandlers=False),
             name="CrochetReactor")
         t.start()
-        self._atexit_register(self._reactor.callFromThread,
-                              self._reactor.stop)
+        self._atexit_register(self._reactor.callFromThread, self._reactor.stop)
         self._atexit_register(_store.log_errors)
         if self._watchdog_thread is not None:
             self._watchdog_thread.start()
@@ -401,44 +390,51 @@ class EventLoop(object):
         If no_setup() is called after setup(), a RuntimeError is raised.
         """
         if self._started:
-            raise RuntimeError("no_setup() is intended to be called once, by a"
-                               " Twisted application, before any libraries "
-                               "using crochet are imported and call setup().")
+            raise RuntimeError(
+                "no_setup() is intended to be called once, by a"
+                " Twisted application, before any libraries "
+                "using crochet are imported and call setup().")
         self._common_setup()
 
     def run_in_reactor(self, function):
         """
-        A decorator that ensures the wrapped function runs in the reactor thread.
+        A decorator that ensures the wrapped function runs in the
+        reactor thread.
 
         When the wrapped function is called, an EventualResult is returned.
         """
-        def runs_in_reactor(result, args, kwargs):
-            d = maybeDeferred(function, *args, **kwargs)
-            result._connect_deferred(d)
+        def _run_in_reactor(wrapped, _, args, kwargs):
+            """
+            Implementation: A decorator that ensures the wrapped function runs in
+            the reactor thread.
+
+            When the wrapped function is called, an EventualResult is returned.
+            """
+
+            if iscoroutinefunction(wrapped):
+                def runs_in_reactor(result, args, kwargs):
+                    d = ensureDeferred(wrapped(*args, **kwargs))
+                    result._connect_deferred(d)
+            else:
+                def runs_in_reactor(result, args, kwargs):
+                    d = maybeDeferred(wrapped, *args, **kwargs)
+                    result._connect_deferred(d)
 
-        @wraps(function)
-        def wrapper(*args, **kwargs):
             result = EventualResult(None, self._reactor)
             self._registry.register(result)
             self._reactor.callFromThread(runs_in_reactor, result, args, kwargs)
             return result
-        wrapper.wrapped_function = function
-        return wrapper
 
-    def wait_for_reactor(self, function):
-        """
-        DEPRECATED, use wait_for(timeout) instead.
+        if iscoroutinefunction(function):
+            # Create a non-async wrapper with same signature.
+            @wraps(function)
+            def non_async_wrapper():
+                pass
+        else:
+            # Just use default behavior of looking at underlying object.
+            non_async_wrapper = None
 
-        A decorator that ensures the wrapped function runs in the reactor thread.
-
-        When the wrapped function is called, its result is returned or its
-        exception raised. Deferreds are handled transparently.
-        """
-        warnings.warn("@wait_for_reactor is deprecated, use @wait_for instead",
-                      DeprecationWarning, stacklevel=2)
-        # This will timeout, in theory. In practice the process will be dead
-        # long before that.
-        return self.wait_for(2 ** 31)(function)
+        return wrapt.decorator(_run_in_reactor, adapter=non_async_wrapper)(function)
 
     def wait_for(self, timeout):
         """
@@ -450,38 +446,33 @@ class EventLoop(object):
         timeout after the given number of seconds (a float), raising a
         crochet.TimeoutError, and cancelling the Deferred being waited on.
         """
+
         def decorator(function):
-            @wraps(function)
-            def wrapper(*args, **kwargs):
+            def wrapper(function, _, args, kwargs):
                 @self.run_in_reactor
                 def run():
-                    return function(*args, **kwargs)
+                    if iscoroutinefunction(function):
+                        return ensureDeferred(function(*args, **kwargs))
+                    else:
+                        return function(*args, **kwargs)
+
                 eventual_result = run()
                 try:
                     return eventual_result.wait(timeout)
                 except TimeoutError:
                     eventual_result.cancel()
                     raise
-            wrapper.wrapped_function = function
-            return wrapper
+
+            if iscoroutinefunction(function):
+                # Create a non-async wrapper with same signature.
+                @wraps(function)
+                def non_async_wrapper():
+                    pass
+            else:
+                # Just use default behavior of looking at underlying object.
+                non_async_wrapper = None
+
+            wrapper = wrapt.decorator(wrapper, adapter=non_async_wrapper)
+            return wrapper(function)
+
         return decorator
-
-    def in_reactor(self, function):
-        """
-        DEPRECATED, use run_in_reactor.
-
-        A decorator that ensures the wrapped function runs in the reactor thread.
-
-        The wrapped function will get the reactor passed in as a first
-        argument, in addition to any arguments it is called with.
-
-        When the wrapped function is called, an EventualResult is returned.
-        """
-        warnings.warn("@in_reactor is deprecated, use @run_in_reactor",
-                      DeprecationWarning, stacklevel=2)
-        @self.run_in_reactor
-        @wraps(function)
-        def add_reactor(*args, **kwargs):
-            return function(self._reactor, *args, **kwargs)
-
-        return add_reactor
