@@ -13,12 +13,23 @@ import weakref
 import tempfile
 import os
 import imp
+import inspect
+from unittest import SkipTest
 
 from twisted.trial.unittest import TestCase
 from twisted.internet.defer import succeed, Deferred, fail, CancelledError
 from twisted.python.failure import Failure
 from twisted.python import threadable
 from twisted.python.runtime import platform
+
+from .._eventloop import (
+    EventLoop, EventualResult, TimeoutError, ResultRegistry, ReactorStopped)
+from .test_setup import FakeReactor
+from .. import (
+    _main, setup, retrieve_result, _store, no_setup,
+    run_in_reactor, wait_for)
+from ..tests import crochet_directory
+
 if platform.type == "posix":
     try:
         from twisted.internet.process import reapAllProcesses
@@ -33,24 +44,18 @@ else:
     # waitpid() is only necessary on POSIX:
     reapAllProcesses = None
 
-from .._eventloop import (EventLoop, EventualResult, TimeoutError,
-                          ResultRegistry, ReactorStopped)
-from .test_setup import FakeReactor
-from .. import (_main, setup, in_reactor, retrieve_result, _store, no_setup,
-                run_in_reactor, wait_for_reactor, wait_for)
-from ..tests import crochet_directory
-
 
 class ResultRegistryTests(TestCase):
     """
     Tests for ResultRegistry.
     """
+
     def test_stopped_registered(self):
         """
         ResultRegistery.stop() fires registered EventualResult with
         ReactorStopped.
         """
-        registry = ResultRegistry(FakeReactor())
+        registry = ResultRegistry()
         er = EventualResult(None, None)
         registry.register(er)
         registry.stop()
@@ -61,7 +66,7 @@ class ResultRegistryTests(TestCase):
         After ResultRegistery.stop() is called subsequent register() calls
         raise ReactorStopped.
         """
-        registry = ResultRegistry(FakeReactor())
+        registry = ResultRegistry()
         er = EventualResult(None, None)
         registry.stop()
         self.assertRaises(ReactorStopped, registry.register, er)
@@ -71,20 +76,20 @@ class ResultRegistryTests(TestCase):
         ResultRegistery.stop() has no impact on registered EventualResult
         which already have a result.
         """
-        registry = ResultRegistry(FakeReactor())
+        registry = ResultRegistry()
         er = EventualResult(succeed(123), None)
         registry.register(er)
         registry.stop()
-        self.assertEqual(er.wait(), 123)
-        self.assertEqual(er.wait(), 123)
-        self.assertEqual(er.wait(), 123)
+        self.assertEqual(er.wait(0.1), 123)
+        self.assertEqual(er.wait(0.1), 123)
+        self.assertEqual(er.wait(0.1), 123)
 
     def test_weakref(self):
         """
         Registering an EventualResult with a ResultRegistry does not prevent
         it from being garbage collected.
         """
-        registry = ResultRegistry(FakeReactor())
+        registry = ResultRegistry()
         er = EventualResult(None, None)
         registry.register(er)
         ref = weakref.ref(er)
@@ -101,6 +106,32 @@ class ResultRegistryTests(TestCase):
         self.assertTrue(ResultRegistry.register.synchronized)
 
 
+def append_in_thread(a_list, f, *args, **kwargs):
+    """
+    Call a function in a thread, append its result to the given list.
+
+    Only return once the thread has actually started.
+
+    Will return a threading.Event that will be set when the action is done.
+    """
+    started = threading.Event()
+    done = threading.Event()
+
+    def go():
+        started.set()
+        try:
+            result = f(*args, **kwargs)
+        except Exception as e:
+            a_list.extend([False, e])
+        else:
+            a_list.extend([True, result])
+        done.set()
+
+    threading.Thread(target=go).start()
+    started.wait()
+    return done
+
+
 class EventualResultTests(TestCase):
     """
     Tests for EventualResult.
@@ -114,7 +145,7 @@ class EventualResultTests(TestCase):
         wait() returns the value the Deferred fired with.
         """
         dr = EventualResult(succeed(123), None)
-        self.assertEqual(dr.wait(), 123)
+        self.assertEqual(dr.wait(0.1), 123)
 
     def test_later_success_result(self):
         """
@@ -122,27 +153,29 @@ class EventualResultTests(TestCase):
         the Deferred is fired after wait() is called.
         """
         d = Deferred()
-        def fireSoon():
-            import time; time.sleep(0.01)
-            d.callback(345)
-        threading.Thread(target=fireSoon).start()
         dr = EventualResult(d, None)
-        self.assertEqual(dr.wait(), 345)
+        result_list = []
+        done = append_in_thread(result_list, dr.wait, 100)
+        time.sleep(0.1)
+        # At this point dr.wait() should have started:
+        d.callback(345)
+        done.wait(100)
+        self.assertEqual(result_list, [True, 345])
 
     def test_success_result_twice(self):
         """
         A second call to wait() returns same value as the first call.
         """
         dr = EventualResult(succeed(123), None)
-        self.assertEqual(dr.wait(), 123)
-        self.assertEqual(dr.wait(), 123)
+        self.assertEqual(dr.wait(0.1), 123)
+        self.assertEqual(dr.wait(0.1), 123)
 
     def test_failure_result(self):
         """
         wait() raises the exception the Deferred fired with.
         """
         dr = EventualResult(fail(RuntimeError()), None)
-        self.assertRaises(RuntimeError, dr.wait)
+        self.assertRaises(RuntimeError, dr.wait, 0.1)
 
     def test_later_failure_result(self):
         """
@@ -150,20 +183,22 @@ class EventualResultTests(TestCase):
         where the Deferred is fired after wait() is called.
         """
         d = Deferred()
-        def fireSoon():
-            time.sleep(0.01)
-            d.errback(RuntimeError())
-        threading.Thread(target=fireSoon).start()
         dr = EventualResult(d, None)
-        self.assertRaises(RuntimeError, dr.wait)
+        result_list = []
+        done = append_in_thread(result_list, dr.wait, 100)
+        time.sleep(0.1)
+        d.errback(RuntimeError())
+        done.wait(100)
+        self.assertEqual(
+            (result_list[0], result_list[1].__class__), (False, RuntimeError))
 
     def test_failure_result_twice(self):
         """
         A second call to wait() raises same value as the first call.
         """
         dr = EventualResult(fail(ZeroDivisionError()), None)
-        self.assertRaises(ZeroDivisionError, dr.wait)
-        self.assertRaises(ZeroDivisionError, dr.wait)
+        self.assertRaises(ZeroDivisionError, dr.wait, 0.1)
+        self.assertRaises(ZeroDivisionError, dr.wait, 0.1)
 
     def test_timeout(self):
         """
@@ -192,8 +227,8 @@ class EventualResultTests(TestCase):
         dr = EventualResult(d, None)
         self.assertRaises(TimeoutError, dr.wait, timeout=0.01)
         d.callback(u"value")
-        self.assertEqual(dr.wait(), u"value")
-        self.assertEqual(dr.wait(), u"value")
+        self.assertEqual(dr.wait(0.1), u"value")
+        self.assertEqual(dr.wait(0.1), u"value")
 
     def test_reactor_thread_disallowed(self):
         """
@@ -211,6 +246,7 @@ class EventualResultTests(TestCase):
         """
         reactor = FakeReactor()
         cancelled = []
+
         def error(f):
             cancelled.append(reactor.in_call_from_thread)
             cancelled.append(f)
@@ -235,8 +271,8 @@ class EventualResultTests(TestCase):
         wrapped by the EventualResult.
         """
         try:
-            1/0
-        except:
+            1 / 0
+        except ZeroDivisionError:
             f = Failure()
         dr = EventualResult(fail(f), None)
         self.assertIdentical(dr.original_failure(), f)
@@ -303,6 +339,8 @@ class EventualResultTests(TestCase):
         If you're wait()ing on an EventualResult in main thread, make sure the
         KeyboardInterrupt happens in timely manner.
         """
+        if platform.type != "posix":
+            raise SkipTest("I don't have the energy to fight Windows semantics.")
         program = """\
 import os, threading, signal, time, sys
 import crochet
@@ -325,21 +363,18 @@ def interrupt():
     # Still running, test shall fail...
     os.kill(os.getpid(), sig_kill)
 
-t = threading.Thread(target=interrupt)
-t.setDaemon(True)
+t = threading.Thread(target=interrupt, daemon=True)
 t.start()
 
 d = Deferred()
 e = crochet.EventualResult(d, None)
 
 try:
-    # Queue.get() has special non-interruptible behavior if not given timeout,
-    # so don't give timeout here.
-    e.wait()
+    e.wait(10000)
 except KeyboardInterrupt:
     sys.exit(23)
 """
-        kw = { 'cwd': crochet_directory }
+        kw = {'cwd': crochet_directory}
         # on Windows the only way to interrupt a subprocess reliably is to
         # create a new process group:
         # http://docs.python.org/2/library/subprocess.html#subprocess.CREATE_NEW_PROCESS_GROUP
@@ -360,7 +395,7 @@ except KeyboardInterrupt:
         er._connect_deferred(d)
         self.assertRaises(TimeoutError, er.wait, 0)
         d.callback(123)
-        self.assertEqual(er.wait(), 123)
+        self.assertEqual(er.wait(0.1), 123)
 
     def test_reactor_stop_unblocks_EventualResult(self):
         """
@@ -454,8 +489,9 @@ except CancelledError:
 else:
     sys.exit(3)
 """
-        process = subprocess.Popen([sys.executable, "-c", program],
-                                   cwd=crochet_directory,)
+        process = subprocess.Popen(
+            [sys.executable, "-c", program],
+            cwd=crochet_directory, )
         self.assertEqual(process.wait(), 23)
 
     def test_noWaitingDuringImport(self):
@@ -470,15 +506,18 @@ else:
         """
         if sys.version_info[0] > 2:
             from unittest import SkipTest
-            raise SkipTest("This test is too fragile (and insufficient) on "
-                           "Python 3 - see "
-                           "https://github.com/itamarst/crochet/issues/43")
+            raise SkipTest(
+                "This test is too fragile (and insufficient) on "
+                "Python 3 - see "
+                "https://github.com/itamarst/crochet/issues/43")
         directory = tempfile.mktemp()
         os.mkdir(directory)
         sys.path.append(directory)
         self.addCleanup(sys.path.remove, directory)
-        with open(os.path.join(directory, "shouldbeunimportable.py"), "w") as f:
-            f.write("""\
+        with open(os.path.join(directory, "shouldbeunimportable.py"),
+                  "w") as f:
+            f.write(
+                """\
 from crochet import EventualResult
 from twisted.internet.defer import Deferred
 
@@ -518,7 +557,7 @@ EventualResult(Deferred(), None).wait(1.0)
         # we want to run .wait while the other thread has the lock acquired
         assertions.append((imp.lock_held(), True))
         try:
-            assertions.append((er.wait(), 123))
+            assertions.append((er.wait(0.1), 123))
         finally:
             test_complete.set()
 
@@ -531,77 +570,24 @@ EventualResult(Deferred(), None).wait(1.0)
         [self.assertEqual(result, expected) for result, expected in assertions]
 
 
-class InReactorTests(TestCase):
-    """
-    Tests for the deprecated in_reactor decorator.
-    """
-
-    def test_name(self):
-        """
-        The function decorated with in_reactor has the same name as the
-        original function.
-        """
-        c = EventLoop(lambda: FakeReactor(), lambda f, g: None)
-
-        @c.in_reactor
-        def some_name(reactor):
-            pass
-        self.assertEqual(some_name.__name__, "some_name")
-
-    def test_in_reactor_thread(self):
-        """
-        The function decorated with in_reactor is run in the reactor
-        thread, and takes the reactor as its first argument.
-        """
-        myreactor = FakeReactor()
-        c = EventLoop(lambda: myreactor, lambda f, g: None)
-        c.no_setup()
-
-        calls = []
-
-        @c.in_reactor
-        def func(reactor, a, b, c):
-            self.assertIdentical(reactor, myreactor)
-            self.assertTrue(reactor.in_call_from_thread)
-            calls.append((a, b, c))
-
-        func(1, 2, c=3)
-        self.assertEqual(calls, [(1, 2, 3)])
-
-    def test_run_in_reactor_wrapper(self):
-        """
-        in_reactor is implemented on top of run_in_reactor.
-        """
-        wrapped = [False]
-
-        def fake_run_in_reactor(function):
-            def wrapper(*args, **kwargs):
-                wrapped[0] = True
-                result = function(*args, **kwargs)
-                wrapped[0] = False
-                return result
-            return wrapper
-
-        myreactor = FakeReactor()
-        c = EventLoop(lambda: myreactor, lambda f, g: None)
-        c.no_setup()
-        c.run_in_reactor = fake_run_in_reactor
-
-
-        @c.in_reactor
-        def func(reactor):
-            self.assertTrue(wrapped[0])
-            return 17
-
-        result = func()
-        self.assertFalse(wrapped[0])
-        self.assertEqual(result, 17)
-
-
 class RunInReactorTests(TestCase):
     """
     Tests for the run_in_reactor decorator.
     """
+
+    def test_signature(self):
+        """
+        The function decorated with the run_in_reactor decorator has the same
+        signature as the original function.
+        """
+        c = EventLoop(lambda: FakeReactor(), lambda f, g: None)
+
+        def some_name(arg1, arg2, karg1=2, *args, **kw):
+            pass
+        decorated = c.run_in_reactor(some_name)
+        self.assertEqual(inspect.signature(some_name),
+                         inspect.signature(decorated))
+
     def test_name(self):
         """
         The function decorated with run_in_reactor has the same name as the
@@ -612,6 +598,7 @@ class RunInReactorTests(TestCase):
         @c.run_in_reactor
         def some_name():
             pass
+
         self.assertEqual(some_name.__name__, "some_name")
 
     def test_run_in_reactor_thread(self):
@@ -632,6 +619,65 @@ class RunInReactorTests(TestCase):
         func(1, 2, c=3)
         self.assertEqual(calls, [(1, 2, 3)])
 
+    def test_method(self):
+        """
+        The function decorated with the wait decorator can be a method.
+        """
+        myreactor = FakeReactor()
+        c = EventLoop(lambda: myreactor, lambda f, g: None)
+        c.no_setup()
+        calls = []
+
+        class C(object):
+            @c.run_in_reactor
+            def func(self, a, b, c):
+                calls.append((self, a, b, c))
+
+        o = C()
+        o.func(1, 2, c=3)
+        self.assertEqual(calls, [(o, 1, 2, 3)])
+
+    def test_classmethod(self):
+        """
+        The function decorated with the wait decorator can be a classmethod.
+        """
+        myreactor = FakeReactor()
+        c = EventLoop(lambda: myreactor, lambda f, g: None)
+        c.no_setup()
+        calls = []
+
+        class C(object):
+            @c.run_in_reactor
+            @classmethod
+            def func(cls, a, b, c):
+                calls.append((cls, a, b, c))
+
+            @classmethod
+            @c.run_in_reactor
+            def func2(cls, a, b, c):
+                calls.append((cls, a, b, c))
+
+        C.func(1, 2, c=3)
+        C.func2(1, 2, c=3)
+        self.assertEqual(calls, [(C, 1, 2, 3), (C, 1, 2, 3)])
+
+    def test_wrap_method(self):
+        """
+        The object decorated with the wait decorator can be a method object
+        """
+        myreactor = FakeReactor()
+        c = EventLoop(lambda: myreactor, lambda f, g: None)
+        c.no_setup()
+        calls = []
+
+        class C(object):
+            def func(self, a, b, c):
+                calls.append((a, b, c))
+
+        f = c.run_in_reactor(C().func)
+        f(4, 5, c=6)
+        self.assertEqual(calls, [(4, 5, 6)])
+
     def make_wrapped_function(self):
         """
         Return a function wrapped with run_in_reactor that returns its first
@@ -644,6 +690,7 @@ class RunInReactorTests(TestCase):
         @c.run_in_reactor
         def passthrough(argument):
             return argument
+
         return passthrough
 
     def test_deferred_success_result(self):
@@ -654,7 +701,7 @@ class RunInReactorTests(TestCase):
         passthrough = self.make_wrapped_function()
         result = passthrough(succeed(123))
         self.assertIsInstance(result, EventualResult)
-        self.assertEqual(result.wait(), 123)
+        self.assertEqual(result.wait(0.1), 123)
 
     def test_deferred_failure_result(self):
         """
@@ -665,7 +712,7 @@ class RunInReactorTests(TestCase):
         passthrough = self.make_wrapped_function()
         result = passthrough(fail(ZeroDivisionError()))
         self.assertIsInstance(result, EventualResult)
-        self.assertRaises(ZeroDivisionError, result.wait)
+        self.assertRaises(ZeroDivisionError, result.wait, 0.1)
 
     def test_regular_result(self):
         """
@@ -675,7 +722,7 @@ class RunInReactorTests(TestCase):
         passthrough = self.make_wrapped_function()
         result = passthrough(123)
         self.assertIsInstance(result, EventualResult)
-        self.assertEqual(result.wait(), 123)
+        self.assertEqual(result.wait(0.1), 123)
 
     def test_exception_result(self):
         """
@@ -688,11 +735,11 @@ class RunInReactorTests(TestCase):
 
         @c.run_in_reactor
         def raiser():
-            1/0
+            1 / 0
 
         result = raiser()
         self.assertIsInstance(result, EventualResult)
-        self.assertRaises(ZeroDivisionError, result.wait)
+        self.assertRaises(ZeroDivisionError, result.wait, 0.1)
 
     def test_registry(self):
         """
@@ -712,30 +759,50 @@ class RunInReactorTests(TestCase):
     def test_wrapped_function(self):
         """
         The function wrapped by @run_in_reactor can be accessed via the
-        `wrapped_function` attribute.
+        `__wrapped__` attribute.
         """
         c = EventLoop(lambda: None, lambda f, g: None)
+
         def func():
             pass
+
         wrapper = c.run_in_reactor(func)
-        self.assertIdentical(wrapper.wrapped_function, func)
+        self.assertIdentical(wrapper.__wrapped__, func)
+
+    def test_async_function(self):
+        """
+        Async functions can be wrapped with @run_in_reactor.
+        """
+        myreactor = FakeReactor()
+        c = EventLoop(lambda: myreactor, lambda f, g: None)
+        c.no_setup()
+        calls = []
+
+        @c.run_in_reactor
+        async def go():
+            self.assertTrue(myreactor.in_call_from_thread)
+            calls.append(1)
+            return 23
+
+        self.assertEqual((go().wait(0.1), go().wait(0.1)), (23, 23))
+        self.assertEqual(len(calls), 2)
+        self.assertFalse(inspect.iscoroutinefunction(go))
 
 
-class WaitTestsMixin(object):
+class WaitTests(TestCase):
     """
-    Tests mixin for the wait_for_reactor/wait_for decorators.
+    Tests for wait_for decorators.
     """
+
     def setUp(self):
         self.reactor = FakeReactor()
         self.eventloop = EventLoop(lambda: self.reactor, lambda f, g: None)
         self.eventloop.no_setup()
 
+    DECORATOR_CALL = "wait_for(timeout=5)"
+
     def decorator(self):
-        """
-        Return a callable that decorates a function, using the decorator being
-        tested.
-        """
-        raise NotImplementedError()
+        return lambda func: self.eventloop.wait_for(timeout=5)(func)
 
     def make_wrapped_function(self):
         """
@@ -743,11 +810,13 @@ class WaitTestsMixin(object):
         its first argument, or raises it if it's an exception.
         """
         decorator = self.decorator()
+
         @decorator
         def passthrough(argument):
             if isinstance(argument, Exception):
                 raise argument
             return argument
+
         return passthrough
 
     def test_name(self):
@@ -756,21 +825,38 @@ class WaitTestsMixin(object):
         original function.
         """
         decorator = self.decorator()
+
         @decorator
         def some_name(argument):
             pass
+
         self.assertEqual(some_name.__name__, "some_name")
 
+    def test_signature(self):
+        """
+        The function decorated with the wait decorator has the same signature
+        as the original function.
+        """
+        decorator = self.decorator()
+
+        def some_name(arg1, arg2, karg1=2, *args, **kw):
+            pass
+        decorated = decorator(some_name)
+        self.assertEqual(inspect.signature(some_name),
+                         inspect.signature(decorated))
+
     def test_wrapped_function(self):
         """
         The function wrapped by the wait decorator can be accessed via the
-        `wrapped_function` attribute.
+        `__wrapped__` attribute.
         """
         decorator = self.decorator()
+
         def func():
             pass
+
         wrapper = decorator(func)
-        self.assertIdentical(wrapper.wrapped_function, func)
+        self.assertIdentical(wrapper.__wrapped__, func)
 
     def test_reactor_thread_disallowed(self):
         """
@@ -813,6 +899,28 @@ class WaitTestsMixin(object):
         func(1, 2, c=3)
         self.assertEqual(calls, [(1, 2, 3)])
 
+    def test_classmethod(self):
+        """
+        The function decorated with the wait decorator can be a classmethod.
+        """
+        calls = []
+        decorator = self.decorator()
+
+        class C(object):
+            @decorator
+            @classmethod
+            def func(cls, a, b, c):
+                calls.append((a, b, c))
+
+            @classmethod
+            @decorator
+            def func2(cls, a, b, c):
+                calls.append((a, b, c))
+
+        C.func(1, 2, c=3)
+        C.func2(1, 2, c=3)
+        self.assertEqual(calls, [(1, 2, 3), (1, 2, 3)])
+
     def test_deferred_success_result(self):
         """
         If the underlying function returns a Deferred, the wrapper returns a
@@ -853,6 +961,8 @@ class WaitTestsMixin(object):
         A call to a decorated function responds to a Ctrl-C (i.e. with a
         KeyboardInterrupt) in a timely manner.
         """
+        if platform.type != "posix":
+            raise SkipTest("I don't have the energy to fight Windows semantics.")
         program = """\
 import os, threading, signal, time, sys
 import crochet
@@ -875,8 +985,7 @@ def interrupt():
     # Still running, test shall fail...
     os.kill(os.getpid(), sig_kill)
 
-t = threading.Thread(target=interrupt)
-t.setDaemon(True)
+t = threading.Thread(target=interrupt, daemon=True)
 t.start()
 
 @crochet.%s
@@ -887,8 +996,8 @@ try:
     wait()
 except KeyboardInterrupt:
     sys.exit(23)
-""" % (self.DECORATOR_CALL,)
-        kw = { 'cwd': crochet_directory }
+""" % (self.DECORATOR_CALL, )
+        kw = {'cwd': crochet_directory}
         if platform.type.startswith('win'):
             kw['creationflags'] = subprocess.CREATE_NEW_PROCESS_GROUP
         process = subprocess.Popen([sys.executable, "-c", program], **kw)
@@ -917,36 +1026,17 @@ try:
     er = run()
 except crochet.ReactorStopped:
     sys.exit(23)
-"""  % (self.DECORATOR_CALL,)
+""" % (self.DECORATOR_CALL, )
         process = subprocess.Popen([sys.executable, "-c", program],
                                    cwd=crochet_directory)
         self.assertEqual(process.wait(), 23)
 
-
-class WaitForReactorTests(WaitTestsMixin, TestCase):
-    """
-    Tests for the wait_for_reactor decorator.
-    """
-    DECORATOR_CALL = "wait_for_reactor"
-
-    def decorator(self):
-        return self.eventloop.wait_for_reactor
-
-
-class WaitForTests(WaitTestsMixin, TestCase):
-    """
-    Tests for the wait_for_reactor decorator.
-    """
-    DECORATOR_CALL = "wait_for(timeout=5)"
-
-    def decorator(self):
-        return lambda func: self.eventloop.wait_for(timeout=5)(func)
-
     def test_timeoutRaises(self):
         """
         If a function wrapped with wait_for hits the timeout, it raises
         TimeoutError.
         """
+
         @self.eventloop.wait_for(timeout=0.5)
         def times_out():
             return Deferred().addErrback(lambda f: f.trap(CancelledError))
@@ -967,21 +1057,39 @@ class WaitForTests(WaitTestsMixin, TestCase):
         @self.eventloop.wait_for(timeout=0.0)
         def times_out():
             return result
+
         self.assertRaises(TimeoutError, times_out)
         self.assertIsInstance(error[0].value, CancelledError)
 
+    def test_async_function(self):
+        """
+        Async functions can be wrapped with @wait_for.
+        """
+        @self.eventloop.wait_for(timeout=0.1)
+        async def go():
+            self.assertTrue(self.reactor.in_call_from_thread)
+            return 17
+
+        self.assertEqual((go(), go()), (17, 17))
+        self.assertFalse(inspect.iscoroutinefunction(go))
+
 
 class PublicAPITests(TestCase):
     """
     Tests for the public API.
     """
+
     def test_no_sideeffects(self):
         """
         Creating an EventLoop object, as is done in crochet.__init__, does not
         call any methods on the objects it is created with.
         """
-        c = EventLoop(lambda: None, lambda f, g: 1/0, lambda *args: 1/0,
-                      watchdog_thread=object(), reapAllProcesses=lambda: 1/0)
+        c = EventLoop(
+            lambda: None,
+            lambda f, g: 1 / 0,
+            lambda *args: 1 / 0,
+            watchdog_thread=object(),
+            reapAllProcesses=lambda: 1 / 0)
         del c
 
     def test_eventloop_api(self):
@@ -994,18 +1102,17 @@ class PublicAPITests(TestCase):
         self.assertIsInstance(_main, EventLoop)
         self.assertEqual(_main.setup, setup)
         self.assertEqual(_main.no_setup, no_setup)
-        self.assertEqual(_main.in_reactor, in_reactor)
         self.assertEqual(_main.run_in_reactor, run_in_reactor)
-        self.assertEqual(_main.wait_for_reactor, wait_for_reactor)
         self.assertEqual(_main.wait_for, wait_for)
         self.assertIdentical(_main._atexit_register, _shutdown.register)
-        self.assertIdentical(_main._startLoggingWithObserver,
-                             startLoggingWithObserver)
+        self.assertIdentical(
+            _main._startLoggingWithObserver, startLoggingWithObserver)
         self.assertIdentical(_main._watchdog_thread, _shutdown._watchdog)
 
     def test_eventloop_api_reactor(self):
         """
-        The publicly exposed EventLoop will, when setup, use the global reactor.
+        The publicly exposed EventLoop will, when setup, use the global
+        reactor.
         """
         from twisted.internet import reactor
         _main.no_setup()
@@ -1025,6 +1132,7 @@ class PublicAPITests(TestCase):
         plaforms.
         """
         self.assertIdentical(_main._reapAllProcesses, reapAllProcesses)
+
     if platform.type != "posix":
         test_reapAllProcesses.skip = "Only relevant on POSIX platforms"
     if reapAllProcesses is None:
