@@ -8,12 +8,12 @@ import threading
 import warnings
 import subprocess
 import sys
+from unittest import SkipTest, TestCase
 
-from twisted.trial.unittest import TestCase
+import twisted
 from twisted.python.log import PythonLoggingObserver
 from twisted.python import log
 from twisted.python.runtime import platform
-from twisted.python import threadable
 from twisted.internet.task import Clock
 
 from .._eventloop import EventLoop, ThreadLogObserver, _store
@@ -72,7 +72,8 @@ class SetupTests(TestCase):
         EventLoop(lambda: reactor, lambda f, *g: None).setup()
         reactor.started.wait(5)
         self.assertNotEqual(reactor.thread_id, None)
-        self.assertNotEqual(reactor.thread_id, threading.current_thread().ident)
+        self.assertNotEqual(
+            reactor.thread_id, threading.current_thread().ident)
         self.assertFalse(reactor.installSignalHandlers)
 
     def test_second_does_nothing(self):
@@ -93,19 +94,20 @@ class SetupTests(TestCase):
         """
         atexit = []
         reactor = FakeReactor()
-        s = EventLoop(lambda: reactor, lambda f, *args: atexit.append((f, args)))
+        s = EventLoop(
+            lambda: reactor, lambda f, *args: atexit.append((f, args)))
         s.setup()
         self.assertEqual(len(atexit), 2)
         self.assertFalse(reactor.stopping)
         f, args = atexit[0]
         self.assertEqual(f, reactor.callFromThread)
-        self.assertEqual(args, (reactor.stop,))
+        self.assertEqual(args, (reactor.stop, ))
         f(*args)
         self.assertTrue(reactor.stopping)
         f, args = atexit[1]
         self.assertEqual(f, _store.log_errors)
         self.assertEqual(args, ())
-        f(*args) # make sure it doesn't throw an exception
+        f(*args)  # make sure it doesn't throw an exception
 
     def test_runs_with_lock(self):
         """
@@ -120,32 +122,36 @@ class SetupTests(TestCase):
         ThreadLogObserver, removing the default log observer.
         """
         logging = []
+
         def fakeStartLoggingWithObserver(observer, setStdout=1):
             self.assertIsInstance(observer, ThreadLogObserver)
             wrapped = observer._observer
             expected = PythonLoggingObserver.emit
             # Python 3 and 2 differ in value of __func__:
             expected = getattr(expected, "__func__", expected)
-            self.assertIdentical(wrapped.__func__, expected)
+            self.assertIs(wrapped.__func__, expected)
             self.assertEqual(setStdout, False)
             self.assertTrue(reactor.in_call_from_thread)
             logging.append(observer)
 
         reactor = FakeReactor()
-        loop = EventLoop(lambda: reactor, lambda f, *g: None,
-                         fakeStartLoggingWithObserver)
+        loop = EventLoop(
+            lambda: reactor, lambda f, *g: None, fakeStartLoggingWithObserver)
         loop.setup()
         self.assertTrue(logging)
         logging[0].stop()
 
     def test_stop_logging_on_exit(self):
         """
-        setup() registers a reactor shutdown event that stops the logging thread.
+        setup() registers a reactor shutdown event that stops the logging
+        thread.
         """
         observers = []
         reactor = FakeReactor()
-        s = EventLoop(lambda: reactor, lambda f, *arg: None,
-                      lambda observer, setStdout=1: observers.append(observer))
+        s = EventLoop(
+            lambda: reactor,
+            lambda f, *arg: None,
+            lambda observer, setStdout=1: observers.append(observer))
         s.setup()
         self.addCleanup(observers[0].stop)
         self.assertIn(("after", "shutdown", observers[0].stop), reactor.events)
@@ -155,15 +161,17 @@ class SetupTests(TestCase):
         setup() ensure the warnings module's showwarning is unmodified,
         overriding the change made by normal Twisted logging setup.
         """
+
         def fakeStartLoggingWithObserver(observer, setStdout=1):
             warnings.showwarning = log.showwarning
             self.addCleanup(observer.stop)
+
         original = warnings.showwarning
         reactor = FakeReactor()
-        loop = EventLoop(lambda: reactor, lambda f, *g: None,
-                         fakeStartLoggingWithObserver)
+        loop = EventLoop(
+            lambda: reactor, lambda f, *g: None, fakeStartLoggingWithObserver)
         loop.setup()
-        self.assertIdentical(warnings.showwarning, original)
+        self.assertIs(warnings.showwarning, original)
 
     def test_start_watchdog_thread(self):
         """
@@ -171,8 +179,8 @@ class SetupTests(TestCase):
         """
         thread = FakeThread()
         reactor = FakeReactor()
-        loop = EventLoop(lambda: reactor, lambda *args: None,
-                         watchdog_thread=thread)
+        loop = EventLoop(
+            lambda: reactor, lambda *args: None, watchdog_thread=thread)
         loop.setup()
         self.assertTrue(thread.started)
 
@@ -185,9 +193,11 @@ class SetupTests(TestCase):
         atexit = []
         thread = FakeThread()
         reactor = FakeReactor()
-        loop = EventLoop(lambda: reactor, lambda f, *arg: atexit.append(f),
-                         lambda observer, *a, **kw: observers.append(observer),
-                         watchdog_thread=thread)
+        loop = EventLoop(
+            lambda: reactor,
+            lambda f, *arg: atexit.append(f),
+            lambda observer, *a, **kw: observers.append(observer),
+            watchdog_thread=thread)
 
         loop.no_setup()
         loop.setup()
@@ -213,9 +223,8 @@ class SetupTests(TestCase):
         reactor = FakeReactor()
         s = EventLoop(lambda: reactor, lambda f, *g: None)
         s.setup()
-        self.assertEqual(reactor.events,
-                         [("before", "shutdown", s._registry.stop)])
-
+        self.assertEqual(
+            reactor.events, [("before", "shutdown", s._registry.stop)])
 
     def test_no_setup_registry_shutdown(self):
         """
@@ -225,23 +234,28 @@ class SetupTests(TestCase):
         reactor = FakeReactor()
         s = EventLoop(lambda: reactor, lambda f, *g: None)
         s.no_setup()
-        self.assertEqual(reactor.events,
-                         [("before", "shutdown", s._registry.stop)])
+        self.assertEqual(
+            reactor.events, [("before", "shutdown", s._registry.stop)])
 
 
 class ProcessSetupTests(TestCase):
     """
     setup() enables support for IReactorProcess on POSIX plaforms.
     """
+
     def test_posix(self):
         """
         On POSIX systems, setup() installs a LoopingCall that runs
         t.i.process.reapAllProcesses() 10 times a second.
         """
+        if platform.type != "posix":
+            raise SkipTest("SIGCHLD is a POSIX-specific issue")
         reactor = FakeReactor()
         reaps = []
-        s = EventLoop(lambda: reactor, lambda f, *g: None,
-                      reapAllProcesses=lambda: reaps.append(1))
+        s = EventLoop(
+            lambda: reactor,
+            lambda f, *g: None,
+            reapAllProcesses=lambda: reaps.append(1))
         s.setup()
         reactor.advance(0.1)
         self.assertEquals(reaps, [1])
@@ -249,71 +263,18 @@ class ProcessSetupTests(TestCase):
         self.assertEquals(reaps, [1, 1])
         reactor.advance(0.1)
         self.assertEquals(reaps, [1, 1, 1])
-    if platform.type != "posix":
-        test_posix.skip = "SIGCHLD is a POSIX-specific issue"
 
     def test_non_posix(self):
         """
-        On POSIX systems, setup() does not install a LoopingCall.
+        On non-POSIX systems, setup() does not install a LoopingCall.
         """
+        if platform.type == "posix":
+            raise SkipTest("This test is for non-POSIX systems.")
         reactor = FakeReactor()
         s = EventLoop(lambda: reactor, lambda f, *g: None)
         s.setup()
         self.assertFalse(reactor.getDelayedCalls())
 
-    if platform.type == "posix":
-        test_non_posix.skip = "SIGCHLD is a POSIX-specific issue"
-
-
-class ThreadLogObserverTest(TestCase):
-    """
-    Tests for ThreadLogObserver.
-    """
-    def test_stop(self):
-        """
-        ThreadLogObserver.stop() stops the thread started in __init__.
-        """
-        threadLog = ThreadLogObserver(None)
-        self.assertTrue(threadLog._thread.is_alive())
-        threadLog.stop()
-        threadLog._thread.join()
-        self.assertFalse(threadLog._thread.is_alive())
-
-    def test_emit(self):
-        """
-        ThreadLogObserver.emit runs the wrapped observer's in its thread, with
-        the given message.
-        """
-        messages = []
-        def observer(msg):
-            messages.append((threading.current_thread().ident, msg))
-
-        threadLog = ThreadLogObserver(observer)
-        ident = threadLog._thread.ident
-        msg1 = {}
-        msg2 = {"a": "b"}
-        threadLog(msg1)
-        threadLog(msg2)
-        threadLog.stop()
-        # Wait for writing to finish:
-        threadLog._thread.join()
-        self.assertEqual(messages, [(ident, msg1), (ident, msg2)])
-
-
-    def test_ioThreadUnchanged(self):
-        """
-        ThreadLogObserver does not change the Twisted I/O thread (which is
-        supposed to match the thread the main reactor is running in.)
-        """
-        threadLog = ThreadLogObserver(None)
-        threadLog.stop()
-        threadLog._thread.join()
-        self.assertIn(threadable.ioThread,
-                      # Either reactor was never run, or run in thread running
-                      # the tests:
-                      (None, threading.current_thread().ident))
-
-
 
 class ReactorImportTests(TestCase):
     """
@@ -324,6 +285,7 @@ class ReactorImportTests(TestCase):
     doesn't work if reactor is imported
     (https://twistedmatrix.com/trac/ticket/7105).
     """
+
     def test_crochet_import_no_reactor(self):
         """
         Importing crochet should not import the reactor.
@@ -338,3 +300,78 @@ if "twisted.internet.reactor" not in sys.modules:
         process = subprocess.Popen([sys.executable, "-c", program],
                                    cwd=crochet_directory)
         self.assertEqual(process.wait(), 23)
+
+
+LOGGING_PROGRAM = """\
+import sys
+from logging import StreamHandler, Formatter, getLogger, DEBUG
+handler = StreamHandler(sys.stdout)
+handler.setFormatter(Formatter("%%(levelname)s %%(message)s"))
+l = getLogger("twisted")
+l.addHandler(handler)
+l.setLevel(DEBUG)
+
+import crochet
+crochet.setup()
+from twisted.python import log
+%s
+log.msg("log-info")
+log.msg("log-error", isError=True)
+"""
+
+
+class LoggingTests(TestCase):
+    """
+    End-to-end tests for Twisted->stdlib logging bridge.
+    """
+    maxDiff = None
+
+    def test_old_logging(self):
+        """
+        Messages from the old Twisted logging API are emitted to Python
+        standard library logging.
+        """
+        if tuple(map(int, twisted.__version__.split("."))) >= (15, 2, 0):
+            raise SkipTest("This test is for Twisted < 15.2.")
+
+        program = LOGGING_PROGRAM % ("", )
+        output = subprocess.check_output([sys.executable, "-u", "-c", program],
+                                         cwd=crochet_directory)
+        self.assertTrue(
+            output.startswith(
+                """\
+INFO Log opened.
+INFO log-info
+ERROR log-error
+"""))
+
+    def test_new_logging(self):
+        """
+        Messages from both new and old Twisted logging APIs are emitted to
+        Python standard library logging.
+        """
+        if tuple(map(int, twisted.__version__.split("."))) < (15, 2, 0):
+            raise SkipTest("This test is for Twisted 15.2 and later.")
+
+        program = LOGGING_PROGRAM % (
+            """\
+from twisted.logger import Logger
+l2 = Logger()
+import time
+time.sleep(1)  # workaround, there is race condition... somewhere
+l2.info("logger-info")
+l2.critical("logger-critical")
+l2.warn("logger-warning")
+l2.debug("logger-debug")
+""", )
+        output = subprocess.check_output([sys.executable, "-u", "-c", program],
+                                         cwd=crochet_directory)
+        self.assertIn(
+            """\
+INFO logger-info
+CRITICAL logger-critical
+WARNING logger-warning
+DEBUG logger-debug
+INFO log-info
+CRITICAL log-error
+""", output.decode("utf-8").replace("\r\n", "\n"))
